@@ -563,6 +563,19 @@ DataConstantsTransformerInlineFormset = inlineformset_factory(DataTransformer, D
 HardwareStatsTransformerInlineFormset = inlineformset_factory(DataTransformer, HardwareStatsDataTransformer, extra=0, formset=BaseInlineFormSetWithAddMoreButton,
                                                               form=HardwareStatsTransformerForm, can_delete_extra=True)
 
+def get_values_for_keys_containing(contains, dict):
+    valid_keys = []
+    for key in dict.keys():
+        if contains in key:
+            valid_keys.append(key)
+
+    sub_dict = {}
+
+    for valid_key in valid_keys:
+        sub_dict[valid_key] = dict[valid_key]
+
+    return sub_dict
+
 from django.utils.safestring import mark_safe
 
 
@@ -582,39 +595,75 @@ class DataTransform(forms.ModelForm):
         print('Init Data Transform')
 
         super(DataTransform, self).__init__(*args, **kwargs)
+        nested_data_transformer_form_prefix = str(self.instance.id) + 'dataTransformerInlineFormset'
+        nested_constants_form_prefix = str(self.instance.id) + 'DataConstantsTransformerInlineFormset'
+        nested_hardware_outputs_form_prefix = str(self.instance.id) + 'HardwareTransformerForm'
+        nested_hardware_stats_form_prefix = str(self.instance.id) + 'HardwareStatsTransformerInlineFormset'
 
+        nested_data_transformer_form_data = None
+        nested_constants_form_data = None
+        nested_hardware_outputs_form_data = None
+        nested_hardware_stats_form_data = None
+
+        if 'data' in kwargs.keys():
+            query_dict = kwargs['data'].dict()
+            form_data = {}
+            form_data['id'] = query_dict['id']
+            form_data['accessory'] = query_dict['accessory']
+            form_data['type'] = query_dict['type']
+
+            nested_data_transformer_form_data = get_values_for_keys_containing(nested_data_transformer_form_prefix,
+                                                                               query_dict)
+            nested_constants_form_data = get_values_for_keys_containing(nested_constants_form_prefix, query_dict)
+            nested_hardware_outputs_form_data = get_values_for_keys_containing(nested_hardware_outputs_form_prefix,
+                                                                               query_dict)
+            nested_hardware_stats_form_data = get_values_for_keys_containing(nested_hardware_stats_form_prefix,
+                                                                             query_dict)
         try:
-            if self.instance.get_children().count() >= 0:
-                print('Adding to child form node: ' + str(self.instance.get_children().count() + 1))
+            if hasattr(self.instance, 'get_children'):
+                print('Adding to root form node: ' + str(self.instance.get_children().count() + 1))
 
                 self.nested_data_transformer_form = dataTransformerInlineFormset(
                     instance=self.instance,
                     queryset=self.instance.get_children(),
                     data=self.data if self.is_bound else None,
                     files=self.files if self.is_bound else None,
-                    initial=[{'parent': str(self.instance.pk),
+                    initial=[{'parent': str(self.instance.pk), 'accessory': str(self.instance.accessory.id),
                               'id': str(uuid.uuid4())}],
-                    prefix=str(self.instance.id) + 'dataTransformerInlineFormset'
+                    prefix=nested_data_transformer_form_prefix,
                 )
                 self.nested_data_transformer_form.extra = self.instance.get_children().count()
 
+            else:
 
+                self.nested_data_transformer_form = dataTransformerInlineFormset(
+                    instance=self.instance,
+                    data=self.data if self.is_bound else None,
+                    files=self.files if self.is_bound else None,
+                    prefix=nested_data_transformer_form_prefix,
+                    initial=[{'parent': str(self.instance.pk),
+                              'id': str(uuid.uuid4())}],
+                )
+                self.nested_data_transformer_form.extra = 0
         except Exception as e:
             print(str(e))
-            # self.nested_data_transformer_form = dataTransformerInlineFormset(
-            #     instance=self.instance,
-            #     data=self.data if self.is_bound else None,
-            #     files=self.files if self.is_bound else None,
-            #     prefix=str(self.instance.id) + 'dataTransformerInlineFormset'
-            # )
-            # self.nested_data_transformer_form.extra = 0
+            if nested_data_transformer_form_data is not None and len(nested_data_transformer_form_data.keys()) > 0:
+                self.nested_data_transformer_form = dataTransformerInlineFormset(
+                    instance=self.instance,
+                    data=self.data if self.is_bound else None,
+                    files=self.files if self.is_bound else None,
+                    prefix=nested_data_transformer_form_prefix,
+                    initial=[{'parent': str(self.instance.pk),
+                              'id': str(uuid.uuid4())}],
+                )
+                self.nested_data_transformer_form.extra = 0
 
         self.nested_constants_form = DataConstantsTransformerInlineFormset(
             instance=self.instance,
             queryset=self.instance.data_transformer_constants,
             data=self.data if self.is_bound else None,
             files=self.files if self.is_bound else None,
-            prefix=str(self.instance.id) + 'DataConstantsTransformerInlineFormset',
+            prefix=nested_constants_form_prefix,
             initial=[{'data_transformer': str(self.instance.pk), 'id': str(uuid.uuid4())}]
 
         )
@@ -625,7 +674,7 @@ class DataTransform(forms.ModelForm):
             instance=hardware_outputs,
             data=self.data if self.is_bound else None,
             files=self.files if self.is_bound else None,
-            prefix=str(self.instance.id) + 'HardwareTransformerForm',
+            prefix=nested_hardware_outputs_form_prefix,
             initial={'data_transformer': str(self.instance.pk), 'id': str(uuid.uuid4())}
         )
 
@@ -634,7 +683,7 @@ class DataTransform(forms.ModelForm):
             queryset=self.instance.hardwarestatsdatatransformer_set,
             data=self.data if self.is_bound else None,
             files=self.files if self.is_bound else None,
-            prefix=str(self.instance.id) + 'HardwareStatsTransformerInlineFormset',
+            prefix=nested_hardware_stats_form_prefix,
             initial=[{'data_transformer': str(self.instance.pk), 'id': str(uuid.uuid4())}]
 
         )
@@ -713,6 +762,7 @@ dataTransformerInlineFormset = inlineformset_factory(DataTransformer, DataTransf
                                                      )
 
 
+
 class DataTransformRoot(forms.ModelForm):
     id = forms.UUIDField(required=False,
                          widget=forms.HiddenInput(attrs={'class': 'form-control'}),initial=uuid.uuid4)
@@ -724,6 +774,30 @@ class DataTransformRoot(forms.ModelForm):
     #     super(HardwareTypeForm, self).__init__(*args, **kwargs)
     def __init__(self, *args, **kwargs):
         super(DataTransformRoot, self).__init__(*args, **kwargs)
+        nested_data_transformer_form_prefix = str(self.instance.id) + 'dataTransformerInlineFormset'
+        nested_constants_form_prefix = str(self.instance.id) + 'DataConstantsTransformerInlineFormset'
+        nested_hardware_outputs_form_prefix = str(self.instance.id) + 'HardwareTransformerForm'
+        nested_hardware_stats_form_prefix = str(self.instance.id) + 'HardwareStatsTransformerInlineFormset'
+
+        nested_data_transformer_form_data = None
+        nested_constants_form_data = None
+        nested_hardware_outputs_form_data = None
+        nested_hardware_stats_form_data = None
+
+        if 'data' in kwargs.keys():
+            query_dict = kwargs['data'].dict()
+            form_data = {}
+            form_data['id'] = query_dict['id']
+            form_data['accessory'] = query_dict['accessory']
+            form_data['type'] = query_dict['type']
+
+            nested_data_transformer_form_data = get_values_for_keys_containing(nested_data_transformer_form_prefix,
+                                                                               query_dict)
+            nested_constants_form_data = get_values_for_keys_containing(nested_constants_form_prefix, query_dict)
+            nested_hardware_outputs_form_data = get_values_for_keys_containing(nested_hardware_outputs_form_prefix,
+                                                                               query_dict)
+            nested_hardware_stats_form_data = get_values_for_keys_containing(nested_hardware_stats_form_prefix,
+                                                                             query_dict)
         # self.fields['accessory'].initial = str(self.instance.accessory.pk)
 
         # types = list(
@@ -740,7 +814,7 @@ class DataTransformRoot(forms.ModelForm):
                     files=self.files if self.is_bound else None,
                     initial=[{'parent': str(self.instance.pk), 'accessory': str(self.instance.accessory.id),
                               'id': str(uuid.uuid4())}],
-                    prefix=str(self.instance.id) + 'dataTransformerInlineFormset',
+                    prefix=nested_data_transformer_form_prefix,
                 )
                 self.nested_data_transformer_form.extra = self.instance.get_children().count()
 
@@ -750,20 +824,31 @@ class DataTransformRoot(forms.ModelForm):
                     instance=self.instance,
                     data=self.data if self.is_bound else None,
                     files=self.files if self.is_bound else None,
-                    prefix=str(self.instance.id) + 'dataTransformerInlineFormset',
+                    prefix=nested_data_transformer_form_prefix,
                     initial=[{'parent': str(self.instance.pk),
                               'id': str(uuid.uuid4())}],
                 )
                 self.nested_data_transformer_form.extra = 0
         except Exception as e:
             print(str(e))
+            if nested_data_transformer_form_data is not None and len(nested_data_transformer_form_data.keys())>0:
+                self.nested_data_transformer_form = dataTransformerInlineFormset(
+                    instance=self.instance,
+                    data=self.data if self.is_bound else None,
+                    files=self.files if self.is_bound else None,
+                    prefix=nested_data_transformer_form_prefix,
+                    initial=[{'parent': str(self.instance.pk),
+                              'id': str(uuid.uuid4())}],
+                )
+                self.nested_data_transformer_form.extra = 0
+
 
         self.nested_constants_form = DataConstantsTransformerInlineFormset(
             instance=self.instance,
             queryset=self.instance.data_transformer_constants,
             data=self.data if self.is_bound else None,
             files=self.files if self.is_bound else None,
-            prefix=str(self.instance.id) + 'DataConstantsTransformerInlineFormset',
+            prefix=nested_constants_form_prefix,
             initial=[{'data_transformer': str(self.instance.pk), 'id': str(uuid.uuid4())}]
 
         )
@@ -774,7 +859,7 @@ class DataTransformRoot(forms.ModelForm):
             instance=hardware_outputs,
             data=self.data if self.is_bound else None,
             files=self.files if self.is_bound else None,
-            prefix=str(self.instance.id) + 'HardwareTransformerForm',
+            prefix=nested_hardware_outputs_form_prefix,
             initial={'data_transformer': str(self.instance.pk), 'id': str(uuid.uuid4())}
         )
 
@@ -783,7 +868,7 @@ class DataTransformRoot(forms.ModelForm):
             queryset=self.instance.hardwarestatsdatatransformer_set,
             data=self.data if self.is_bound else None,
             files=self.files if self.is_bound else None,
-            prefix=str(self.instance.id) + 'HardwareStatsTransformerInlineFormset',
+            prefix=nested_hardware_stats_form_prefix,
             initial=[{'data_transformer': str(self.instance.pk), 'id': str(uuid.uuid4())}]
 
         )
